function [filenames, filepaths, idxOpts] = parsefileinputs(optList, filetype, varargin)
% PARSEFILEINPUTS Parses the input and checks their validity.
%   Returns the filenames and filepaths for the input files, as well as the
%   index where the remaining options start.
%
% -----
%
% Usage:
%   [filenames, filepaths, idxOpts] = PARSEPOLARINPUTS (optList, filetype)
%   [filenames, filepaths, idxOpts] = PARSEPOLARINPUTS (optList, filetype, ...)

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

import af_tools.utils.*

% First ensures the option list and filetype is properly set
validateattributes(optList, {'char','string','cell'},{}, mfilename(), 'optList', 1)
validateattributes(filetype, {'char','string'},{'scalartext'}, mfilename(), 'filetype', 2)
filetype = char(filetype);
if filetype(1) ~= '.'
    error('MATLAB:parsefileinputs:invalidFileType', 'The filetype must starts by a point (.dat, .m, .mat, etc). Found ''%s''!\n', filetype);
end

% Constants and defaults
DEFAULT_INPUTFILES = '*';

% Parse inputs
hasNoInput = isempty(varargin) || isoption(optList, varargin{1});
if hasNoInput
    % Prompt user to select files
    ext = ['*',filetype];
    [filenames, filepaths] = uigetfile(ext, 'Select all dat-files to aggregate', 'MultiSelect', 'on');
    idxOpts = 1;
else
    inputDir = varargin{1};
    idxOpts = 2;
    if nargin >= 4 && ~isoption(optList, varargin{2})
        idxOpts = 3;
        inputFiles = varargin{2};
    else
        inputFiles = DEFAULT_INPUTFILES;
    end
    
    % Validate inputDir
    validateattributes(inputDir, {'char','string'},{'nonempty','vector'}, mfilename(), 'inputDir')
    
    
    if ~exist(inputDir, 'dir')
        error('MATLAB:parsefileinputs:dirNotFound', 'The directory specified as inputDir (%s) can not be found!\n', inputDir);
    end
    
    % Validate inputFiles
    validateattributes(inputFiles, {'char','cell'},{'nonempty','vector'}, mfilename(), 'inputFiles', 4)
    
    if iscell(inputFiles) &&  ~iscellstr(inputFiles)
        error('MATLAB:parsefileinputs:wrongInputFiles', ['If inpuFiles is given as a cell array, it must '...
            'contain only character vectors!\n']);
    end
    
    % ------------------------------------------------------
    % Look for files
    
    % Get absolute path
    if contains(inputDir, pwd)
        filepaths = inputDir;
    else
        filepaths = fullfile(pwd, inputDir);
    end
    
    % Convert inputFiles to string for simpler handling
    inputFiles = string(inputFiles);
    
    % Get all files
    AllFiles = [];
    
    for i = 1:length(inputFiles)
        inputFiles(i) = appendextension(inputFiles(i),filetype); % Add extension
        dummy = dir(fullfile(filepaths,inputFiles(i)));
        if isempty(dummy)
            warning('MATLAB:parsefileinputs:FileNotFound', 'Could not find file %s.', fullfile(filepaths,inputFiles(i)));
        end
        AllFiles = [AllFiles; dummy];
    end
    
    if isempty(AllFiles)
        error('MATLAB:parsefileinputs:noFilesFound', 'Impossible to find ANY file matching the input arguments.');
    end
    
    filenames = cell(1,length(AllFiles));
    for i = 1:length(AllFiles)
        filenames{i} = AllFiles(i).name;
    end
    
end


end


% ------------------------------------------------------------------------------
function bool = isoption(optList, var)
% ISOPTION Returns true if var is contained in option list

if iscell(var)
    bool = false;
else
    bool = any(strcmpi(var,optList));
end
end

