function [alpha, cl, cd, cm, idxOpts] = parsepolarinputs(optList, varargin)
% PARSEPOLARINPUTS Parses the input and checks their validity.
%   Returns the arrays for the main polar arrays (alpha, cl, cd, cm), as well as
%   the index where the remaining options start.
%   If some inputs were not provided (alpha, cl, cd, cm), the functions returns
%   an empty array for those.
%
%   This function also contains checks on every "polar" input (i.e. the Polar
%   strcture or the four arrays: alpha, cl, cd and cm) to ensure they are
%   correct.
% -----
%
% Usage:
%   [alpha, cl, cd, cm, idxOpts] = PARSEPOLARINPUTS (optList)
%   [alpha, cl, cd, cm, idxOpts] = PARSEPOLARINPUTS (optList, POLAR)
%   [...] = PARSEPOLARINPUTS (optList, ALPHA)
%   [...] = PARSEPOLARINPUTS (optList, ALPHA, CL, ...)
%   [...] = PARSEPOLARINPUTS (optList, ALPHA, CL, CD, ...)
%   [...] = PARSEPOLARINPUTS (optList, ALPHA, CL, CD, CM, ...)

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

import af_tools.utils.*

% First ensures the option list is properly set
validateattributes(optList, {'char','string','cell'},{}, mfilename(), 'optList', 1)

% Initialize outputs
alpha = [];
cl = [];
cd = [];
cm = [];

% Parse inputs
hasNoInput = isempty(varargin) || isoption(optList, varargin{1});
if hasNoInput
    idxOpts = 1;
    % Prompt user for polar
    [filename, path] = uigetfile('*.mat', 'Select a Polar file');
    tmp = load([path,filename]);
    [alpha, cl, cd, cm] = extractpolar(tmp.Polar);
    clear tmp;
else
    if isstruct(varargin{1})
        idxOpts = 2;
        % Extract polar values
        Polar = varargin{1};
        validateattributes(Polar, {'struct'},{'nonempty'}, mfilename(), 'Polar', 1)
        [alpha, cl, cd, cm] = extractpolar(varargin{1});
    else
        % Parse user-provided arrays
        if length(varargin) >= 1
            [alpha, idxOpts, foundOpts] = assignvar(optList, alpha, varargin, 1);
            if length(varargin) >= 2 && ~foundOpts
                [cl, idxOpts, foundOpts] = assignvar(optList, cl, varargin, 2);
                if length(varargin) >= 3 && ~foundOpts
                    [cd, idxOpts, foundOpts] = assignvar(optList, cd, varargin, 3);
                    if length(varargin) >= 4 && ~foundOpts
                        [cm, idxOpts, ~] = assignvar(optList, cm, varargin, 4);
                    end
                end
            end
        end
    end
end

% Transform row vectors into column vectors
alpha = vecttocol(alpha);
cl = vecttocol(cl);
cd = vecttocol(cd);
cm = vecttocol(cm);

% Validate input arrays or inputs coming from Polar structure
validateattributes(alpha, {'numeric'},{'real', '2d', 'nonempty','increasing'}, mfilename(), 'alpha', 1)

if ~isempty(cl)
    validateattributes(cl, {'numeric'},{'real', 'nrows', size(alpha,1), 'nonempty'}, mfilename(), 'cl', 2)
end
if ~isempty(cd)
    validateattributes(cd, {'numeric'},{'real', 'nrows', size(alpha,1), 'nonempty'}, mfilename(), 'cd', 3)
end
if ~isempty(cm)
    validateattributes(cm, {'numeric'},{'real', 'nrows', size(alpha,1), 'nonempty'}, mfilename(), 'cm', 4)
end

% Standardize alpha size so we have one column per Polar as well
if size(alpha,2) == 1 && ~isempty(cl)
    alpha = repmat(alpha, [1, size(cl,2)]);
end

end


% ------------------------------------------------------------------------------
function bool = isoption(optList, var)
% ISOPTION Returns true if var is contained in option list

bool = any(strcmpi(var,optList));
end


% ------------------------------------------------------------------------------
function [alpha, cl, cd, cm] = extractpolar(Polar)
% EXTRACTPOLAR Extract alpha, cl, cd and cm fields Polar structure.

alpha = Polar.alpha;
cl = Polar.cl;
cd = Polar.cd;
cm = Polar.cm;
end

% ------------------------------------------------------------------------------
function [var, idxOpts, foundOpts] = assignvar(optList, var, data, idx)
% ASSIGNVAR Assign varargin value to a variable, ouptut corresponding idxOpts.

foundOpts = false;
if ~isoption(optList, data{idx})
    idxOpts = idx+1;
    var = data{idx};
else
    idxOpts = idx;
    foundOpts = true;
end

end
