function [alpha_zeroL, cd_zeroL] = findzerolift(varargin)
% FINDZEROLIFT Finds the zero-lift angle and the associated cd.
%   This function finds the zero-lift angle by interpolating around the last
%   negative and first positive values for the lift coefficient.
%
%   FINDZEROLIFT accepts inputs in the form of a Polar structure (generated with
%   XF2MAT) or values for angles of attack and their associated cl and cd.
%
%   If the input is specified as aoa, cl and cd, the three arrays must have the
%   same number of rows (one for each angle of attack). The columns indicate
%   values for different polars (different Mach, different Reynolds,...).
%   For example, if the cl values are given for 2 different Reynols and there
%   are 100 values for the aoa, cl and cd must be [100 x 2].
%
%   If extended polars are used as input (see EXTENDPOLAR), the function will
%   only look for the zero-lift angle in the range of [-30, 30] deg.
%
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   [ALPHA_ZEROL, CD_ZEROL] = FINDZEROLIFT prompts the user to input a Polar
%   structure and determines the zero-lift angle of attack (ALPHA_ZEROL) and the
%   drag coefficient at this angle (CD_ZEROL).
%
%   [ALPHA_ZEROL, CD_ZEROL] = FINDZEROLIFT(POLAR) uses the POLAR strcture given
%   as input instead of prompting the user to select the file.
%
%   [ALPHA_ZEROL, CD_ZEROL] = FINDZEROLIFT(ALPHA, CL) uses directly the values
%   given in in ALPHA and CL to determine the zero-lift angle. As no CD is
%   provided, CD_ZEROL will be an empty vector.
%
%   [ALPHA_ZEROL, CD_ZEROL] = FINDZEROLIFT(ALPHA, CL, CD) uses directly the
%   values given in in ALPHA and CL to determine the zero-lift angle. As CD is
%   provided, CD_ZEROL will be returned as well.
%
% Inputs:
%   Polar  : Polar structure (e.g. output from XF2MAT)
%   alpha  : Vector or array with angles of attack [aoas x 1] or [aoas x nbPolar]
%   cl     : Array with lift coefficients [aoas x nbPolar]
%   cd     : Array with drag coefficients [aoas x nbPolar]
%
% Output:
%   alpha_zeroL : Zero-lift angle of attack, IN RADIANS
%   cd_zeroL    : Drag at zero-lift angle of attack
%
% Example:
%   FINDZEROLIFT
%   FINDZEROLIFT(Polar)
%   FINDZEROLIFT(alpha, cl)
%   FINDZEROLIFT(alpha, cl, cd)
%
% See also: XF2MAT, FINDSTALL, FINDCLLINEARRANGE.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

narginchk(0,3);

% Import other functions from this package
import af_tools.utils.*

% Defaults and constants
ORIGINAL_RANGE = deg2rad([-30;30]);  % Look for values in this range only
OPTION_LIST = {};   % No options for this function

% Extract and validate the inputs
[alpha, cl, cd] = parsepolarinputs(OPTION_LIST,varargin{:});

% Trim extended polars to original range
% (avoid issues if extended polars are used as input)
[alpha, cl, cd] = trimtorange(ORIGINAL_RANGE, alpha, cl, cd);

%------------------------------------
% Find zero lift for each polar

alpha_zeroL = zeros(1,size(cl,2));
cd_zeroL = zeros(1,size(cl,2));
if nargin == 2
    for i = 1:size(cl,2)
        alpha_zeroL(i) = findzeroliftvect(alpha, cl(:,i));
    end
    nar
else
    for i = 1:size(cl,2)
        [alpha_zeroL(i), cd_zeroL(i)] = findzeroliftvect(alpha(:,i), cl(:,i), cd(:,i));
    end
end

end


%--------------------------------------------------------------------------
function [alpha_zeroL, cd_zeroL] = findzeroliftvect(alpha, cl, cd)
% FINDZEROLIFTVECT Finds the zero-lift point and returns the corresponding AOA and CD
%   For vectors!

% Process:
%   1. Get the value of the last negative and first positive Cl
%   2. Use linear interpolation between the two

PLOT_POLAR_IF_ERROR = true; % Plot the polar if error so user can see why it fails

idx = find(cl(2:end) > 0, 1);  % Find first cl > 0
idx = idx + 1; % Needs a +1 because we looked for cl(2:end)

if isempty(idx)
    if PLOT_POLAR_IF_ERROR
        figure('Name', 'Debug: Polar with no zero lift')
        plot(rad2deg(alpha), cl)
        title('Polar with no clear zero lift angle')
        xlabel('AOA [deg]')
        ylabel('C_l [-]')
        grid on
    end
    error('MATLAB:findzerolift:noPositiveCl',['Impossible to find a '...
        'positive Cl']);
end

if sign(cl(idx)) == sign(cl(idx-1))
    if PLOT_POLAR_IF_ERROR
        figure('Name', 'Debug: Polar with no zero lift')
        plot(rad2deg(alpha), cl)
        title('Polar with no clear zero lift angle')
        xlabel('AOA [deg]')
        ylabel('C_l [-]')
        grid on
    end
    error('MATLAB:findzerolift:noNegativeCl',['Impossible to find a '...
        'negative Cl']);
end

dummy(:,1) = alpha(idx-1:idx); % AOA of the two points around cl = 0
dummy(:,2) = cl(idx-1:idx); % Cl of the two points around cl = 0
alpha_zeroL = interp1(dummy(:,2),dummy(:,1), 0);

if nargin == 3
    cd_zeroL = interp1(alpha, cd, alpha_zeroL);
else
    cd_zeroL = NaN;
end

end
