function Dat = uiuccleaner(varargin)
% UIUCCLEANER Cleans airfoil data from UIUC Airfoil Database.
%   The UIUC airfoil database from the University of Illinois gathers the
%   coordinates for more than 1600 airfoils. However, there is a few
%   discrepancies between the format and ordering of these coordinates. This
%   script alleviates that by re-formatting any input file from the original
%   database.
%
% Format:
%   The output format of this script follows the "labeled coordinates" file,
%   used originally by Selig on the UIUC Airfoil Database.
%   The first line contains the name of the airfoil, and each line after that is
%   a set of coordinates. The coordinates are ordered from the trailing edge,
%   along the upper surface to the leading edge and back around the lower
%   surface to trailing edge.
%
% Note:
%   This function can be used on many files at once in order to speed-up the
%   process.
%
% <a href="https://gitlab.uliege.be/thlamb/airfoil_toolbox">Documentation (README)</a>
% <a href="https://gitlab.uliege.be/thlamb/airfoil_toolbox/-/issues">Report an issue</a>
% -----
%
% Usage:
%   DAT = UIUCCLEANER prompts the user for all inputs, then converts the
%   coordinates to the correct standard.
%
%   DAT = UIUCCLEANER(INPUTDIR) converts all dat files found in INPUTDIR.
%
%   DAT = UIUCCLEANER(INPUTDIR, INPUTFILES) converts only the polars
%   specified by INPUTFILES found in INPUTDIR.
%
%   DAT = UIUCCLEANER(..., 'autosave', true) saves the resulting coordinates
%   in a new dat-file.
%
%   DAT = UIUCCLEANER(..., 'overwrite', true) saves the resulting coordinates
%   by overwriting the original dat-file. If autosave is not specified but
%   overwrite is true, the file will be overwritten anyway. If autosave is set
%   to false and overwrite is true, the file will not be saved at all.
%
%   DAT = UIUCCLEANER(..., 'refine', true) refines the input data by using a
%   spline interpolation with 100 coordinates in total, spaced following the
%   half-cosine rule. False by default.
%
% Inputs:
%   inputDir   : Path of the directory with the UIUC oridinal dat-files
%   inputFiles : Files to select (ex: '*' (default), '*0012*', {'file1','file2'}, etc)
%
% Output:
%   Dat : A structure for the results for each input file.
%         - Dat.file : original filename
%         - Dat.airfoil : airfoil name, parsed from the input file
%         - Dat.x : airfoil X coordinates, with 0 <= X <= 1
%         - Dat.y : airfoil Y coordinates.
%
% Example:
%   UIUCCLEANER
%   UIUCCLEANER('test_data')
%   UIUCCLEANER('test_data', '*0012*')
%   UIUCCLEANER('test_data', {'0012_re1e5', '0012_re1e6'})
%   UIUCCLEANER('test_data', 'autosave', true)
%   UIUCCLEANER('test_data', 'overwrite', true)
%   UIUCCLEANER('test_data', 'refine', true)
%
% See also: NACAAIRFOIL.

% -----
% UIUC Airfoil Database: https://m-selig.ae.illinois.edu/ads/coord_database.html
% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------

narginchk(0,8);

% Import other functions from this package
import af_tools.utils.*

% Constants and defaults
OPTION_LIST = {'autosave', 'overwrite', 'refine'};
FILETYPE = '.dat';
REFINED_SPACING = 'halfcosine';
REFINED_NPOINTS = ceil((100+1)/2);

% Parse inputs
[allFileNames, fullpath, idxOpts] = parsefileinputs(OPTION_LIST, FILETYPE, varargin{:});
[autosave, overwrite, refine] = parseoptioninputs(varargin{idxOpts:end});

% Convert filenames to string array
allFileNames = string(allFileNames);

% -------------------------------------
% Convert all files

% Number of files found
nbFiles=length(allFileNames);

% Initialize structures
Dat = struct('file', [], 'airfoil', [], 'x', [], 'y', []);

for i=1:nbFiles
    
    % Load file and start parsing
    % No matter the file type, line 1 should always be the airfoil name
    fileID = fopen(fullfile(fullpath,allFileNames{i}));
    tmpAirfoil = textscan(fileID,'%s', 1, 'Delimiter','\n:','HeaderLines',0);
    resultArray = cell2mat(textscan(fileID,'%f %f %*[^\n]', 'HeaderLines',0));
    fclose(fileID); clear fileID;
    
    % Re-order file if it is needed
    if resultArray(1,1) ~= 0 && resultArray(1,1) ~= 1 && resultArray(2,1) == 0
        % Remove first line
        resultArray = resultArray(2:end,:);
        
        % Find index at the end of the upper side (normally diff would be -1,
        % but we use -0.5 in case the LE and TE are not precisely at 0 and 1. It
        % should be the only negative diff anyway).
        idx = find(diff(resultArray) < -0.5 ,1);
        upper = flipud(resultArray(1:idx,:));
        lower = resultArray(idx+1:end,:);
        resultArray = [upper;lower(2:end,:)];
    end
    
    % Refine coordinates to add more points
    if refine && size(resultArray,1) < 2*REFINED_NPOINTS
        xc = fliplr(spacedvector(REFINED_SPACING, REFINED_NPOINTS))';
        
        idx = find(diff(resultArray(:,1)) < 0, 1, 'last');
        upper = resultArray(1:idx+1,:);
        lower = resultArray(idx+1:end,:);
        
        refinedUpperY = interp1(upper(:,1),upper(:,2),xc,'spline');
        refinedLowerY = interp1(lower(:,1),lower(:,2),flipud(xc),'spline');
        
        x = [xc; flipud(xc(1:end-1))];
        y = [refinedUpperY; refinedLowerY(2:end)];
        resultArray = [x,y];
    end
    
    % Output structure
    Dat(i).path = fullpath;
    Dat(i).file = allFileNames{i};
    Dat(i).airfoil = char(cellstr(tmpAirfoil{:}));
    Dat(i).x = resultArray(:,1);
    Dat(i).y = resultArray(:,2);
    
end


% -------------------------------------
% Save results automatically

if autosave
    
    for i = 1:length(Dat)
        
        filename = split(Dat(i).file,'.dat');
        if ~overwrite
            filename = [filename{1} ,'-CLEAN'];
        end
        filename = appendextension(filename,'.dat');
        filename = fullfile(Dat(i).path,filename);
        header = Dat(i).airfoil;
        savetodat(filename, header, Dat(i).x, Dat(i).y)
        
    end
    
end

end


% ------------------------------------------------------------------------------
function [autosave, overwrite, refine] = parseoptioninputs(varargin)
% PARSEOPTIONINPUTS Parses the options and checks their validity

import af_tools.utils.*

% Constants and defaults
DEFAULT_AUTOSAVE = false;
DEFAULT_OVEWRITE = false;
DEFAULT_REFINE = false;

% Option validator
validLogical = @(x) validateattributes(x, {'logical'},{'scalar'}, mfilename());

% Parse options
p = inputParser;
addParameter(p,'autosave',DEFAULT_AUTOSAVE, validLogical);
addParameter(p,'overwrite',DEFAULT_OVEWRITE, validLogical);
addParameter(p,'refine',DEFAULT_REFINE, validLogical);
parse(p,varargin{:});
autosave = p.Results.autosave;
overwrite = p.Results.overwrite;
refine = p.Results.refine;

end
