% TEST_PARSEFILEINPUTS Unitary tests for the parsefileinputs sub-function
% Note:
%   Matlab package functionality is not very test-framework friendly.
%   Either the whole package has to be imported in EVERY SINGLE TEST or the
%   functions must be called as <package>.function() everytime.
%   The second option is preferred for the tests as it has the smaller scope.

% -----
% Copyright 2022 Thomas Lambert <t.lambert@uliege.be>
% ULiege - Aeroelasticity and Experimental Aerodynamics
% Apache 2.0 License
% https://gitlab.uliege.be/am-dept/matlab_airfoil_toolbox
% ------------------------------------------------------------------------------


%% Main test function

function tests = test_parsefileinputs

tests = functiontests(localfunctions);

end


%% Setup and teardown

function setupOnce(testCase)

addpath('../.'); % Add repository to Matlab Path
addpath('./test_utils'); % Add utils to Matlab Path

% Set random number generator settings
testCase.TestData.currentRNG = rng;

end

function teardownOnce(testCase)

rmpath('../.'); % Remove repository from Matlab Path
rmpath('./test_utils'); % Remove utils from Matlab Path

% Restore the random number generator settings
s = testCase.TestData.currentRNG;
rng(s)

end

function teardown(~)

close all; % Close all figures that would have been openend

end


%% Check if errors and warnings are raised

function test_invalidOptsList(testCase)
% Error if optList is not a cell array or a string array


empty = [];
scal = 0;
logi = true;

% OptList must be a char, string or cell array
verifyError(testCase, @() af_tools.utils.parsefileinputs(empty), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(scal), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(logi), 'MATLAB:parsefileinputs:invalidType')

end

function test_invalidFiletype(testCase)
% Error if filetype does not start with a point

EMPTY_OPTS = {};
wrongFt = 'dat';

% OptList must be a char, string or cell array
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS,wrongFt), 'MATLAB:parsefileinputs:invalidFileType')

end


function test_invalidInputTypeWhenNoOpts(testCase)
% Error if any input is of the wrong type

EMPTY_OPTS = {};
FILETYPE = '.dat';
empty = [];
scal = 0;
logi = true;
Struct = struct;

% 1. inputDir (char vector or string)
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, empty), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, scal), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, logi), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, Struct), 'MATLAB:parsefileinputs:invalidType')

% 2. inputFile (char, string, cell of chars)
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, scal, empty), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, scal, scal), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, scal, logi), 'MATLAB:parsefileinputs:invalidType')
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, scal, Struct), 'MATLAB:parsefileinputs:invalidType')

end


% -------------------------------------
function test_dirNotFound(testCase)
% Error if valid inputDir not found

EMPTY_OPTS = {};
FILETYPE = '.dat';

verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, 'missingdir'), 'MATLAB:parsefileinputs:dirNotFound')

end


function test_fileNotFound(testCase)
% Error if valid inputDir not found

EMPTY_OPTS = {'test'};
FILETYPE = '.dat';
testdir = [pwd,'/test_utils'];

verifyWarning(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, testdir, {'clarky.dat','testdummy.dat'}), 'MATLAB:parsefileinputs:FileNotFound')
warning('off','MATLAB:parsefileinputs:FileNotFound') % remove warning first warning about FileNotFound before it throws the error
verifyError(testCase, @() af_tools.utils.parsefileinputs(EMPTY_OPTS, FILETYPE, testdir, 'randomFile'), 'MATLAB:parsefileinputs:noFilesFound')
warning on % Reactivate warnings

end

